/*
m_scale_midi - Quantise MIDI notes to a given scale

DESCRIPTION
Quantise chromatic MIDI notes to a given scale with a given root note.
Scales are described in a simple, but long table. Every note will sound, but
notes outside the scale will be remapped to one of the closest notes in
the scale.

SYNTAX
kMappedNote m_scale_midi kInputNote, kScale, kRootNote

INITIALIZATION

PERFORMANCE
kMappedNote - The quantised MIDI note
kInputNote - Input MIDI note (0-127)
kScale - Number of a scale function. A scale, scale names can be retrieved
	with m_scale_get_name
kRootNote - The root note of the scale to use, in the range 0-11, 0=C, 11=B

CREDITS
Author: Jeanette C.
*/

opcode m_scale_midi, k, KKK
	kInputNote, kScale, kRoot xin

	; Initialisation of parameters
	kInputNote init -1
	kScale init -1
	kRoot init -1

	; Internal variables
	iNumberOfScales = ftlen(giScales) / 12
	kLocalScale[] init 12 ; transposed scale array
	kCurNote init 0 ; currently read note from scale
	kCurOffset init 0 ; current scale offset
	kFullScale[] init 128 ; full mapping across the MIDI range
	kOutputNote init 0 ; MIDI note mapped/quantised to scale
	kNoteIndex init 0 ; index into the twelve note scale
	kOctIndex init 0 ; Index into octaves across the range

	; Only if an argument changes, recalculate the scale
	if (changed(kScale, kRoot) != 0) then ; scale or root note changed
		; Is it a valid scale number
		if (kScale < iNumberOfScales) then
			kCurOffset = kScale * 12 ; offset into the scales ftable
			kRoot = wrap(int(kRoot), 0, 11)
			; Transpose the scale description
			kNoteIndex = 0
			while (kNoteIndex < 12) do
				kCurNote = ((tab((kNoteIndex + kCurOffset), giScales, 0) + kRoot) % 12)
				kLocalScale[kNoteIndex] = kCurNote
				kNoteIndex += 1
			od
			kLocalScale = sorta(kLocalScale)

			; Map scale across full MIDI range
			kNoteIndex = 0
			kOctIndex = 0
			while (kNoteIndex <128) do
				kFullScale[kNoteIndex] = kLocalScale[(kNoteIndex % 12)] + (kOctIndex * 12)
				if ((kNoteIndex % 12) == 11) then ; increase octave
					kOctIndex += 1
				endif
				kNoteIndex += 1
			od
		endif
	endif

	; Map incoming MIDI note to scale
	kOutputNote = kFullScale[kInputNote]

	; Output the scale
	xout(kOutputNote)
endop
